@@ -27,20 +27,22 @@
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif /* HAVE_CONFIG_H */
+#endif /* defined(HAVE_CONFIG_H) */
 
-void test_ngtcp2_get_varint(void);
-void test_ngtcp2_get_uvarintlen(void);
-void test_ngtcp2_put_uvarintlen(void);
-void test_ngtcp2_get_uint64(void);
-void test_ngtcp2_get_uint48(void);
-void test_ngtcp2_get_uint32(void);
-void test_ngtcp2_get_uint24(void);
-void test_ngtcp2_get_uint16(void);
-void test_ngtcp2_get_uint16be(void);
-void test_ngtcp2_nth_server_bidi_id(void);
-void test_ngtcp2_nth_server_uni_id(void);
-void test_ngtcp2_nth_client_bidi_id(void);
-void test_ngtcp2_nth_client_uni_id(void);
+#define MUNIT_ENABLE_ASSERT_ALIASES
 
-#endif /* NGTCP2_CONV_TEST_H */
+#include "munit.h"
+
+extern const MunitSuite conv_suite;
+
+munit_void_test_decl(test_ngtcp2_get_varint);
+munit_void_test_decl(test_ngtcp2_get_uvarintlen);
+munit_void_test_decl(test_ngtcp2_put_uvarintlen);
+munit_void_test_decl(test_ngtcp2_get_uint64be);
+munit_void_test_decl(test_ngtcp2_get_uint32be);
+munit_void_test_decl(test_ngtcp2_get_uint24be);
+munit_void_test_decl(test_ngtcp2_get_uint16be);
+munit_void_test_decl(test_ngtcp2_get_uint16);
+munit_void_test_decl(test_ngtcp2_put_pkt_num);
+
+#endif /* !defined(NGTCP2_CONV_TEST_H) */
