@@ -27,11 +27,17 @@
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif /* HAVE_CONFIG_H */
+#endif /* defined(HAVE_CONFIG_H) */
 
-void test_ngtcp2_gaptr_push(void);
-void test_ngtcp2_gaptr_is_pushed(void);
-void test_ngtcp2_gaptr_drop_first_gap(void);
-void test_ngtcp2_gaptr_get_first_gap_after(void);
+#define MUNIT_ENABLE_ASSERT_ALIASES
 
-#endif /* NGTCP2_GAPTR_TEST_H */
+#include "munit.h"
+
+extern const MunitSuite gaptr_suite;
+
+munit_void_test_decl(test_ngtcp2_gaptr_push);
+munit_void_test_decl(test_ngtcp2_gaptr_is_pushed);
+munit_void_test_decl(test_ngtcp2_gaptr_drop_first_gap);
+munit_void_test_decl(test_ngtcp2_gaptr_get_first_gap_after);
+
+#endif /* !defined(NGTCP2_GAPTR_TEST_H) */
