@@ -27,9 +27,16 @@
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif /* HAVE_CONFIG_H */
+#endif /* defined(HAVE_CONFIG_H) */
 
-void test_ngtcp2_encode_ipv4(void);
-void test_ngtcp2_encode_ipv6(void);
+#define MUNIT_ENABLE_ASSERT_ALIASES
 
-#endif /* NGTCP2_STR_TEST_H */
+#include "munit.h"
+
+extern const MunitSuite str_suite;
+
+munit_void_test_decl(test_ngtcp2_encode_ipv4);
+munit_void_test_decl(test_ngtcp2_encode_ipv6);
+munit_void_test_decl(test_ngtcp2_get_bytes);
+
+#endif /* !defined(NGTCP2_STR_TEST_H) */
