@@ -27,10 +27,17 @@
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif /* HAVE_CONFIG_H */
+#endif /* defined(HAVE_CONFIG_H) */
 
-void test_ngtcp2_strm_streamfrq_pop(void);
-void test_ngtcp2_strm_streamfrq_unacked_offset(void);
-void test_ngtcp2_strm_streamfrq_unacked_pop(void);
+#define MUNIT_ENABLE_ASSERT_ALIASES
 
-#endif /* NGTCP2_STRM_TEST_H */
+#include "munit.h"
+
+extern const MunitSuite strm_suite;
+
+munit_void_test_decl(test_ngtcp2_strm_streamfrq_pop);
+munit_void_test_decl(test_ngtcp2_strm_streamfrq_unacked_offset);
+munit_void_test_decl(test_ngtcp2_strm_streamfrq_unacked_pop);
+munit_void_test_decl(test_ngtcp2_strm_discard_reordered_data);
+
+#endif /* !defined(NGTCP2_STRM_TEST_H) */
